from soultrip.constants import *
from soultrip.integrator import Friction, Integrator
from soultrip.data import loadimage
from soultrip.stageobject import StageObject
from euclid import Vector2
from pygame import draw
from pygame import gfxdraw
from pygame import Surface
from pygame import Rect
import pygame
import collections
import math

PLAYER_SURFACE_WIDTH = 700
FACE_WIDTH = 40    
BODY_SIZE = 60

PLAYER_ACCEL = 20
PLAYER_MAX_SPEED = 35

class Player(StageObject):
    def __init__(self):
        StageObject.__init__(self)
        self.pos = Vector2(614, 638)
        self.extents = (BODY_SIZE, BODY_SIZE)
        self.surf_templ = Surface((PLAYER_SURFACE_WIDTH, PLAYER_SURFACE_WIDTH), pygame.SRCALPHA).convert_alpha()
        self.surface = self.surf_templ.copy()
        self.alpha_factor = .3
        self.base_alpha = .3
        
        self.body = Surface((BODY_SIZE, BODY_SIZE))#GameSprite('ghost.png', 1, (200,300), False)
        #draw.circle(self.body, (255,255,255,255 * self.alpha_factor), (20,20), 20)
        self.faces = {
            'default': loadimage('small-ghost-face.png').convert_alpha(),
            'hit': loadimage('ghost-face-hit-small.png').convert_alpha(),
            'lose': loadimage('ghost-face-failure-small.png').convert_alpha(),
            'win': loadimage('ghost-face-success-small.png').convert_alpha()
        }
        #self.faces = {k: pygame.transform.scale(s, (FACE_WIDTH, FACE_WIDTH)) for k, s in self.faces.items()}
        self.show_face = 'default'

        #self.angle = 90 #up

        self.trail = collections.deque(maxlen=15)
        self.trail.append(self.pos)
        self.prev_pos = self.pos
        self.velocity = Vector2(0, -11)

        self.r = 255
        self.g = 255
        self.b = 255
        
        self.rect = None
        self.recalculate_rect()

        self.up_down = 0
        self.left_right = 0
        self.prev_direction = Vector2(0, -1).normalize()
        
        self.hurt = False
        self.hurt_color = False
        self.hurt_timer = 0
        self.hit_invuln_time = 8

        self.got_orb = False
        self.got_orb_timer = 0
        self.brighten_time = 5

        self.brighten = False

        self.orbs = 0

        self.hidden = False
        self.waiting = False

    def hide(self):
        self.hidden = True

    def show(self):
        self.hidden = False

    def update(self, dt):
        if self.waiting:
            return
        StageObject.update(self, dt)

        direction = Vector2(self.left_right, self.up_down).normalize()
        moving = True
        if not direction:
            direction = Vector2(0,0)
            moving = False
        else:
            self.prev_direction = direction

        direction *= PLAYER_ACCEL * dt

        if self.hurt:
            self.show_face = 'hit'
            self.hurt_timer += dt
            self.hurt_color = not self.hurt_color
            self.base_alpha = .3
            if self.hurt_timer > self.hit_invuln_time:
                self.hurt_color = False
                self.hurt = False
                self.hurt_timer = 0
                self.show_face = 'default'

        if self.base_alpha > .8:
            self.show_face = 'win'


        if self.hurt_color:
            self.g = 50
            self.b = 50
        else:
            self.g = 255
            self.b = 255

        #speed constraint
        if self.velocity.magnitude() > PLAYER_MAX_SPEED:
            self.velocity.normalize()
            self.velocity *= PLAYER_MAX_SPEED

        #sides and top / bottom
        if self.pos.x < WALL_SIZE:
            self.pos.x = WALL_SIZE
            self.velocity.x = 0
        if self.pos.x > RESOLUTION_X - WALL_SIZE:
            self.pos.x = RESOLUTION_X - WALL_SIZE
            self.velocity.x = 0

        if self.pos.y + 40 > RESOLUTION_Y:
            self.pos.y = RESOLUTION_Y - 40
            self.velocity.y = 0
        if self.pos.y < 20:
            self.pos.y = 20
            self.velocity.y = 0

        self.pos, self.velocity = Integrator(direction).integrate(self.pos, self.velocity, dt)

        self.trail.appendleft(self.pos)
        self.prev_pos = self.pos

        if self.got_orb:
            self.got_orb_timer += dt
            if self.alpha_factor < .9:
                self.alpha_factor += .1 * dt
            if self.got_orb_timer >= self.brighten_time:
                self.got_orb = False
                self.got_orb_timer = 0
                if self.base_alpha < 1.0:
                    self.base_alpha += 0.1 
        else:
            if self.alpha_factor > self.base_alpha:
                self.alpha_factor -= .1 * dt
            #self.alpha_factor = self.base_alpha

    def draw(self, layer):
        if self.hidden:
            return
        #StageObject.draw(self, layer)
        self.surface = self.surf_templ.copy()

        center = Vector2(PLAYER_SURFACE_WIDTH/2, PLAYER_SURFACE_WIDTH/2)


        color = (self.r, self.g, self.b, min(int(math.floor(255 * self.alpha_factor)), 255))

        width_max = float(BODY_SIZE)
        width = width_max
        for i, vec in enumerate(self.trail):
            vec = center + (vec - self.pos)
            prev_vec = None
            if i == 0:
                prev_vec = center
            else:
                prev_vec = center + (self.trail[i - 1] - self.pos)
            if i < self.trail.maxlen - 1:
                x = int(vec.x)
                y = int(vec.y)
                r = int(width/2.0)
                aa_circle(self.surface, x, y, r, color)
            width -= width_max / self.trail.maxlen

        aa_circle(self.surface, center.x, center.y, BODY_SIZE/2, color)

        x = self.pos.x - PLAYER_SURFACE_WIDTH/2.0
        y = self.pos.y - PLAYER_SURFACE_WIDTH/2.0
        layer.blit(blurSurf(self.surface, 3), (x, y))
        offset_x = self.left_right * 3
        offset_y = self.up_down * 3
        x = self.pos.x - FACE_WIDTH/2.0 + offset_x
        y = self.pos.y - FACE_WIDTH/2.0 + 5 + offset_y
        layer.blit(self.faces[self.show_face], (x, y))


def aa_circle(surface, x, y, r, color):
    #gfxdraw.aacircle(surface, x, y, r, color)
    gfxdraw.filled_circle(surface, x, y, r, color)

def blurSurf(surface, amt):
    scale = 1.0/float(amt)
    surf_size = surface.get_size()
    scale_size = (int(surf_size[0]*scale), int(surf_size[1]*scale))
    surf = pygame.transform.smoothscale(surface, scale_size)
    surf = pygame.transform.smoothscale(surf, surf_size)
    return surf